@@ -1,5 +1,5 @@
 /* apps/ciphers.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
@@ -59,6 +59,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef NO_STDIO
+#define APPS_WIN16
+#endif
 #include "apps.h"
 #include "err.h"
 #include "ssl.h"
@@ -101,7 +104,7 @@ char **argv;
 	apps_startup();
 
 	if (bio_err == NULL)
-		if ((bio_err=BIO_new_fp(stderr,BIO_NOCLOSE)) != NULL)
+		bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
 	STDout=BIO_new_fp(stdout,BIO_NOCLOSE);
 
 	argc--;
