@@ -1,9 +1,9 @@
 /* apps/verify.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -60,7 +60,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "apps.h"
-#include "buffer.h"
+#include "bio.h"
 #include "err.h"
 #include "x509.h"
 #include "pem.h"
@@ -69,9 +69,8 @@
 #define PROG	verify_main
 
 #ifndef NOPROTO
-static int MS_CALLBACK cb(int ok, X509 *xs, X509 *xi, int depth, int error,
-	char *arg);
-static int check(CERTIFICATE_CTX *ctx,char *file);
+static int MS_CALLBACK cb(int ok, X509_STORE_CTX *ctx);
+static int check(X509_STORE *ctx,char *file);
 #else
 static int MS_CALLBACK cb();
 static int check();
@@ -85,17 +84,20 @@ char **argv;
 	{
 	int i,ret=1;
 	char *CApath=NULL,*CAfile=NULL;
-	CERTIFICATE_CTX *cert_ctx=NULL;
+	X509_STORE *cert_ctx=NULL;
+	X509_LOOKUP *lookup=NULL;
 
-	cert_ctx=CERTIFICATE_CTX_new();
+	cert_ctx=X509_STORE_new();
 	if (cert_ctx == NULL) goto end;
+	X509_STORE_set_verify_cb_func(cert_ctx,cb);
+
 	ERR_load_crypto_strings();
 
 	apps_startup();
 
 	if (bio_err == NULL)
 		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
-			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
+			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE|BIO_FP_TEXT);
 
 	argc--;
 	argv++;
@@ -128,13 +130,16 @@ char **argv;
 			break;
 		}
 
-	if ((!X509_load_verify_locations(cert_ctx,CAfile,CApath)) ||
-		(!X509_set_default_verify_paths(cert_ctx)))
-		{
-		BIO_printf(bio_err,"X509_load_verify_locations\n");
-		ERR_print_errors(bio_err);
-		goto end;
-		}
+	lookup=X509_STORE_add_lookup(cert_ctx,X509_LOOKUP_file());
+	if (lookup == NULL) abort();
+	if (!X509_LOOKUP_load_file(lookup,CAfile,X509_FILETYPE_PEM))
+		X509_LOOKUP_load_file(lookup,NULL,X509_FILETYPE_DEFAULT);
+		
+	lookup=X509_STORE_add_lookup(cert_ctx,X509_LOOKUP_hash_dir());
+	if (lookup == NULL) abort();
+	if (!X509_LOOKUP_add_dir(lookup,CApath,X509_FILETYPE_PEM))
+		X509_LOOKUP_add_dir(lookup,NULL,X509_FILETYPE_DEFAULT);
+
 
 	if (argc < 1) check(cert_ctx,NULL);
 	else
@@ -144,17 +149,18 @@ char **argv;
 end:
 	if (ret == 1)
 		BIO_printf(bio_err,"usage: verify [-verbose] [-CApath path] [-CAfile file] cert1 cert2 ...\n");
-	if (cert_ctx != NULL) CERTIFICATE_CTX_free(cert_ctx);
+	if (cert_ctx != NULL) X509_STORE_free(cert_ctx);
 	EXIT(ret);
 	}
 
 static int check(ctx,file)
-CERTIFICATE_CTX *ctx;
+X509_STORE *ctx;
 char *file;
 	{
 	X509 *x=NULL;
 	BIO *in=NULL;
 	int i=0,ret=0;
+	X509_STORE_CTX csc;
 
 	in=BIO_new(BIO_s_file());
 	if (in == NULL)
@@ -183,7 +189,11 @@ char *file;
 		goto end;
 		}
 	fprintf(stdout,"%s: ",(file == NULL)?"stdin":file);
-	i=X509_cert_verify(ctx,x,cb,NULL,NULL);
+
+	X509_STORE_CTX_init(&csc,ctx,x,NULL);
+	i=X509_verify_cert(&csc);
+	X509_STORE_CTX_cleanup(&csc);
+
 	ret=0;
 end:
 	if (i)
@@ -199,35 +209,32 @@ end:
 	return(ret);
 	}
 
-static int MS_CALLBACK cb(ok, xs, xi, depth, error, arg)
+static int MS_CALLBACK cb(ok,ctx)
 int ok;
-X509 *xs;
-X509 *xi;
-int depth;
-int error;
-char *arg;
+X509_STORE_CTX *ctx;
 	{
+	char buf[256];
+
 	if (!ok)
 		{
-		char *s;
 		/* since we are just checking the certificates, it is
 		 * ok if they are self signed. */
-		if (error == VERIFY_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
+		if (ctx->error == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
 			ok=1;
 		else
 			{
-			s=X509_NAME_oneline(X509_get_subject_name(xs));
-			printf("%s\n",s);
-			Free(s);
-			printf("error %d at %d depth lookup:%s\n",error,depth,
-				X509_cert_verify_error_string(error));
+			X509_NAME_oneline(
+				X509_get_subject_name(ctx->current_cert),buf,256);
+			printf("%s\n",buf);
+			printf("error %d at %d depth lookup:%s\n",ctx->error,
+				ctx->error_depth,
+				X509_verify_cert_error_string(ctx->error));
+			if (ctx->error == X509_V_ERR_CERT_HAS_EXPIRED)
+				ok=1;
 			}
 		}
 	if (!v_verbose)
 		ERR_clear_error();
-#ifdef LINT
-	xi=xs; xs=xi;
-#endif
 	return(ok);
 	}
 
