@@ -1,5 +1,5 @@
 /* crypto/asn1/p7_lib.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
@@ -81,11 +81,11 @@ unsigned char **pp;
 		return((int)a->length);
 		}
 
-	ret+=4; /* sequence */
+	ret+=4; /* sequence, BER header plus '0 0' end padding */
 	M_ASN1_I2D_len(a->type,i2d_ASN1_OBJECT);
 	if (a->d.ptr != NULL)
 		{
-		ret+=4; /* explicit tag [ 0 ] */
+		ret+=4; /* explicit tag [ 0 ] BER plus '0 0' */
 		switch (OBJ_obj2nid(a->type))
 			{
 		case NID_pkcs7_data:
@@ -182,6 +182,8 @@ long length;
 			goto err;
 			}
 
+		ret->detached=0;
+
 		c.q=c.p;
 		Tinf=ASN1_get_object(&c.p,&Tlen,&Ttag,&Tclass,
 			(c.inf & 1)?(length+ *pp-c.q):c.slen);
@@ -195,6 +197,8 @@ long length;
 			break;
 		case NID_pkcs7_signed:
 			M_ASN1_D2I_get(ret->d.sign,d2i_PKCS7_SIGNED);
+			if (ret->d.sign->contents->d.ptr == NULL)
+				ret->detached=1;
 			break;
 		case NID_pkcs7_enveloped:
 			M_ASN1_D2I_get(ret->d.enveloped,d2i_PKCS7_ENVELOPE);
@@ -223,6 +227,9 @@ long length;
 				}
 			}
 		}
+	else
+		ret->detached=1;
+		
 	M_ASN1_D2I_Finish(a,PKCS7_free,ASN1_F_D2I_PKCS7);
 	}
 
@@ -234,6 +241,7 @@ PKCS7 *PKCS7_new()
 	ret->type=ASN1_OBJECT_new();
 	ret->asn1=NULL;
 	ret->length=0;
+	ret->detached=0;
 	ret->d.ptr=NULL;
 	return(ret);
 	M_ASN1_New_Error(ASN1_F_PKCS7_NEW);
@@ -244,6 +252,17 @@ PKCS7 *a;
 	{
 	if (a == NULL) return;
 
+	PKCS7_content_free(a);
+	if (a->type != NULL)
+		{
+		ASN1_OBJECT_free(a->type);
+		}
+	Free((char *)(char *)a);
+	}
+
+void PKCS7_content_free(a)
+PKCS7 *a;
+	{
 	if (a->asn1 != NULL) Free((char *)a->asn1);
 
 	if (a->d.ptr != NULL)
@@ -275,8 +294,6 @@ PKCS7 *a;
 			break;
 			}
 		}
-	if (a->type != NULL)
-		ASN1_OBJECT_free(a->type);
-	Free((char *)(char *)a);
+	a->d.ptr=NULL;
 	}
 
