@@ -1,9 +1,9 @@
 /* crypto/asn1/x_name.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -68,6 +68,12 @@
  * ASN1err(ASN1_F_X509_NAME_ENTRY_NEW,ASN1_R_UNKNOWN_ATTRIBUTE_TYPE);
  */
 
+#ifndef NOPROTO
+static int i2d_X509_NAME_entries(X509_NAME *a);
+#else
+static int i2d_X509_NAME_entries();
+#endif
+
 int i2d_X509_NAME_ENTRY(a,pp)
 X509_NAME_ENTRY *a;
 unsigned char **pp;
@@ -103,17 +109,40 @@ long length;
 int i2d_X509_NAME(a,pp)
 X509_NAME *a;
 unsigned char **pp;
+	{
+	int ret;
+
+	if (a == NULL) return(0);
+	if (a->modified)
+		{
+		ret=i2d_X509_NAME_entries(a);
+		if (ret < 0) return(ret);
+		}
+
+	ret=a->bytes->length;
+	if (pp != NULL)
+		{
+		memcpy(*pp,a->bytes->data,ret);
+		*pp+=ret;
+		}
+	return(ret);
+	}
+
+static int i2d_X509_NAME_entries(a)
+X509_NAME *a;
 	{
 	X509_NAME_ENTRY *ne,*fe=NULL;
+	STACK *sk;
+	BUF_MEM *buf=NULL;
 	int set=0,r,ret=0;
 	int i;
 	unsigned char *p;
 	int size=0;
 
-	if (a == NULL) return(0);
-	for (i=0; i<sk_num(a); i++)
+	sk=a->entries;
+	for (i=0; i<sk_num(sk); i++)
 		{
-		ne=(X509_NAME_ENTRY *)sk_value(a,i);
+		ne=(X509_NAME_ENTRY *)sk_value(sk,i);
 		if (fe == NULL)
 			{
 			fe=ne;
@@ -135,12 +164,18 @@ unsigned char **pp;
 	if (fe != NULL)
 		fe->size=size;
 
-	M_ASN1_I2D_seq_total();
+	r=ASN1_object_size(1,ret,V_ASN1_SEQUENCE);
+
+	buf=a->bytes;
+	if (!BUF_MEM_grow(buf,r)) goto err;
+	p=(unsigned char *)buf->data;
+
+	ASN1_put_object(&p,1,ret,V_ASN1_SEQUENCE,V_ASN1_UNIVERSAL);
 
 	set= -1;
-	for (i=0; i<sk_num(a); i++)
+	for (i=0; i<sk_num(sk); i++)
 		{
-		ne=(X509_NAME_ENTRY *)sk_value(a,i);
+		ne=(X509_NAME_ENTRY *)sk_value(sk,i);
 		if (set != ne->set)
 			{
 			set=ne->set;
@@ -149,8 +184,10 @@ unsigned char **pp;
 			}
 		i2d_X509_NAME_ENTRY(ne,&p);
 		}
-
-	M_ASN1_I2D_finish();
+	a->modified=0;
+	return(r);
+err:
+	return(-1);
 	}
 
 X509_NAME *d2i_X509_NAME(a,pp,length)
@@ -158,15 +195,17 @@ X509_NAME **a;
 unsigned char **pp;
 long length;
 	{
-	int set=0;
-	int index=0;
+	int set=0,i;
+	int idx=0;
+	unsigned char *orig;
 	M_ASN1_D2I_vars(a,X509_NAME *,X509_NAME_new);
 
-	if (sk_num(ret) > 0)
+	orig= *pp;
+	if (sk_num(ret->entries) > 0)
 		{
-		while (sk_num(ret) > 0)
+		while (sk_num(ret->entries) > 0)
 			X509_NAME_ENTRY_free((X509_NAME_ENTRY *)
-				sk_pop(ret));
+				sk_pop(ret->entries));
 		}
 
 	M_ASN1_D2I_Init();
@@ -174,13 +213,21 @@ long length;
 	for (;;)
 		{
 		if (M_ASN1_D2I_end_sequence()) break;
-		M_ASN1_D2I_get_set(ret,d2i_X509_NAME_ENTRY);
-		for (; index < sk_num(ret); index++)
+		M_ASN1_D2I_get_set(ret->entries,d2i_X509_NAME_ENTRY);
+		for (; idx < sk_num(ret->entries); idx++)
 			{
-			((X509_NAME_ENTRY *)sk_value(ret,index))->set=set;
+			((X509_NAME_ENTRY *)sk_value(ret->entries,idx))->set=
+				set;
 			}
 		set++;
 		}
+
+	i=(int)(c.p-orig);
+	if (!BUF_MEM_grow(ret->bytes,i)) goto err;
+	memcpy(ret->bytes->data,orig,i);
+	ret->bytes->length=i;
+	ret->modified=0;
+
 	M_ASN1_D2I_Finish(a,X509_NAME_free,ASN1_F_D2I_X509_NAME);
 	}
 
@@ -188,7 +235,10 @@ X509_NAME *X509_NAME_new()
 	{
 	X509_NAME *ret=NULL;
 
-	if ((ret=sk_new(NULL)) == NULL) goto err2;
+	M_ASN1_New_Malloc(ret,X509_NAME);
+	if ((ret->entries=sk_new(NULL)) == NULL) goto err2;
+	M_ASN1_New(ret->bytes,BUF_MEM_new);
+	ret->modified=1;
 	return(ret);
 	M_ASN1_New_Error(ASN1_F_X509_NAME_NEW);
 	}
@@ -206,10 +256,12 @@ X509_NAME_ENTRY *X509_NAME_ENTRY_new()
 	M_ASN1_New_Error(ASN1_F_X509_NAME_ENTRY_NEW);
 	}
 
-void X509_NAME_free(sk)
-X509_NAME *sk;
+void X509_NAME_free(a)
+X509_NAME *a;
 	{
-	sk_pop_free(sk,X509_NAME_ENTRY_free);
+	BUF_MEM_free(a->bytes);
+	sk_pop_free(a->entries,X509_NAME_ENTRY_free);
+	Free((char *)a);
 	}
 
 void X509_NAME_ENTRY_free(a)
@@ -221,3 +273,23 @@ X509_NAME_ENTRY *a;
 	Free((char *)a);
 	}
 
+int X509_NAME_set(xn,name)
+X509_NAME **xn;
+X509_NAME *name;
+	{
+	X509_NAME *in;
+
+	if (*xn == NULL) return(0);
+
+	if (*xn != name)
+		{
+		in=X509_NAME_dup(name);
+		if (in != NULL)
+			{
+			X509_NAME_free(*xn);
+			*xn=in;
+			}
+		}
+	return(*xn != NULL);
+	}
+	
