@@ -1,9 +1,9 @@
 /* crypto/bn/bn_exp.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -58,7 +58,7 @@
 
 #include <stdio.h>
 #include "cryptlib.h"
-#include "bn.h"
+#include "bn_lcl.h"
 
 /* slow but works */
 int BN_mod_mul(ret, a, b, m, ctx)
@@ -83,7 +83,7 @@ err:
 	return(r);
 	}
 
-#ifdef undef
+#if 0
 /* this one works - simple but works */
 int BN_mod_exp(r,a,p,m,ctx)
 BIGNUM *r,*a,*p,*m;
@@ -98,9 +98,9 @@ BN_CTX *ctx;
 	if (BN_copy(v,a) == NULL) goto err;
 	bits=BN_num_bits(p);
 
-	if (p->d[0]&1)
+	if (BN_is_odd(p))
 		{ if (BN_copy(r,a) == NULL) goto err; }
-	else	{ if (BN_one(r) == NULL) goto err; }
+	else	{ if (BN_one(r)) goto err; }
 
 	for (i=1; i<bits; i++)
 		{
@@ -120,6 +120,38 @@ err:
 
 #endif
 
+/* this one works - simple but works */
+int BN_exp(r,a,p,ctx)
+BIGNUM *r,*a,*p;
+BN_CTX *ctx;
+	{
+	int i,bits,ret=0;
+	BIGNUM *v,*tmp;
+
+	v=ctx->bn[ctx->tos++];
+	tmp=ctx->bn[ctx->tos++];
+
+	if (BN_copy(v,a) == NULL) goto err;
+	bits=BN_num_bits(p);
+
+	if (BN_is_odd(p))
+		{ if (BN_copy(r,a) == NULL) goto err; }
+	else	{ if (BN_one(r)) goto err; }
+
+	for (i=1; i<bits; i++)
+		{
+		if (!BN_sqr(tmp,v,ctx)) goto err;
+		if (BN_is_bit_set(p,i))
+			{
+			if (!BN_mul(tmp,r,v)) goto err;
+			}
+		}
+	ret=1;
+err:
+	ctx->tos-=2;
+	return(ret);
+	}
+
 int BN_mod_exp(r,a,p,m,ctx)
 BIGNUM *r;
 BIGNUM *a;
@@ -130,8 +162,14 @@ BN_CTX *ctx;
 	int ret;
 
 #ifdef MONT_MUL_MOD
-	if ((m->d[m->top-1]&BN_TBIT) && (m->d[0]&1))
-		{ ret=BN_mod_exp_mont(r,a,p,m,ctx); }
+	/* I have finally been able to take out this pre-condition of
+	 * the top bit being set.  It was caused by an error in BN_div
+	 * with negatives.  There was also another problem when for a^b%m
+	 * a >= m.  eay 07-May-97 */
+/*	if ((m->d[m->top-1]&BN_TBIT) && BN_is_odd(m)) */
+
+	if (BN_is_odd(m))
+		{ ret=BN_mod_exp_mont(r,a,p,m,ctx,NULL); }
 	else
 #endif
 #ifdef RECP_MUL_MOD
@@ -160,6 +198,11 @@ BN_CTX *ctx;
 	aa=ctx->bn[ctx->tos++];
 	bits=BN_num_bits(p);
 
+	if (bits == 0)
+		{
+		BN_one(r);
+		return(1);
+		}
 	nb=BN_reciprocal(d,m,ctx);
 	if (nb == -1) goto err;
 
@@ -255,34 +298,63 @@ err:
 /* #endif */
 
 /* #ifdef MONT_MUL_MOD */
-int BN_mod_exp_mont(r,a,p,m,ctx)
+int BN_mod_exp_mont(r,a,p,m,ctx,in_mont)
 BIGNUM *r;
 BIGNUM *a;
 BIGNUM *p;
 BIGNUM *m;
 BN_CTX *ctx;
+BN_MONT_CTX *in_mont;
 	{
+#define TABLE_SIZE	16
 	int i,j,bits,ret=0,wstart,wend,window,wvalue;
 	int start=1;
-	BIGNUM *d;
-	BIGNUM *val[16];
+	BIGNUM *d,*aa;
+	BIGNUM *val[TABLE_SIZE];
 	BN_MONT_CTX *mont=NULL;
 
+	if (!(m->d[0] & 1))
+		{
+		BNerr(BN_F_BN_MOD_EXP_MONT,BN_R_CALLED_WITH_EVEN_MODULUS);
+		return(0);
+		}
 	d=ctx->bn[ctx->tos++];
 	bits=BN_num_bits(p);
+	if (bits == 0)
+		{
+		BN_one(r);
+		return(1);
+		}
 
-	if ((mont=BN_MONT_CTX_new()) == NULL) goto err;
-	if (!BN_MONT_CTX_set(mont,m,ctx)) goto err;
+	/* If this is not done, things will break in the montgomery
+	 * part */
+
+#if 1
+	if (in_mont != NULL)
+		mont=in_mont;
+	else
+#endif
+		{
+		if ((mont=BN_MONT_CTX_new()) == NULL) goto err;
+		if (!BN_MONT_CTX_set(mont,m,ctx)) goto err;
+		}
 
 	val[0]=BN_new();
-	if (!BN_to_montgomery(val[0],a,mont,ctx)) goto err; /* 1 */
+	if (BN_ucmp(a,m) >= 0)
+		{
+		BN_mod(val[0],a,m,ctx);
+		aa=val[0];
+		}
+	else
+		aa=a;
+	if (!BN_to_montgomery(val[0],aa,mont,ctx)) goto err; /* 1 */
 	if (!BN_mod_mul_montgomery(d,val[0],val[0],mont,ctx)) goto err; /* 2 */
 
-	if (bits <= 17) /* This is probably 3 or 0x10001, so just do singles */
+	if (bits <= 20) /* This is probably 3 or 0x10001, so just do singles */
 		window=1;
-	else if (bits >= 256)
+	else if (bits > 250)
 		window=5;	/* max size of window */
-	else if (bits >= 128)
+	else if (bits >= 120)
 		window=4;
 	else
 		window=3;
@@ -294,7 +366,7 @@ BN_CTX *ctx;
 		if (!BN_mod_mul_montgomery(val[i],val[i-1],d,mont,ctx))
 			goto err;
 		}
-	for (; i<16; i++)
+	for (; i<TABLE_SIZE; i++)
 		val[i]=NULL;
 
 	start=1;	/* This is used to avoid multiplication etc
@@ -304,14 +376,16 @@ BN_CTX *ctx;
 	wstart=bits-1;	/* The top bit of the window */
 	wend=0;		/* The bottom bit of the window */
 
-        if (!BN_to_montgomery(r,BN_value_one,mont,ctx)) goto err;
+        if (!BN_to_montgomery(r,BN_value_one(),mont,ctx)) goto err;
 	for (;;)
 		{
 		if (BN_is_bit_set(p,wstart) == 0)
 			{
 			if (!start)
+				{
 				if (!BN_mod_mul_montgomery(r,r,r,mont,ctx))
 				goto err;
+				}
 			if (wstart == 0) break;
 			wstart--;
 			continue;
@@ -357,9 +431,9 @@ BN_CTX *ctx;
 	BN_from_montgomery(r,r,mont,ctx);
 	ret=1;
 err:
-	if (mont != NULL) BN_MONT_CTX_free(mont);
+	if ((in_mont == NULL) && (mont != NULL)) BN_MONT_CTX_free(mont);
 	ctx->tos--;
-	for (i=0; i<16; i++)
+	for (i=0; i<TABLE_SIZE; i++)
 		if (val[i] != NULL) BN_clear_free(val[i]);
 	return(ret);
 	}
@@ -381,6 +455,12 @@ BN_CTX *ctx;
 	d=ctx->bn[ctx->tos++];
 	bits=BN_num_bits(p);
 
+	if (bits == 0)
+		{
+		BN_one(r);
+		return(1);
+		}
+
 	val[0]=BN_new();
 	if (!BN_mod(val[0],a,m,ctx)) goto err;		/* 1 */
 	if (!BN_mod_mul(d,val[0],val[0],m,ctx))
