@@ -1,9 +1,9 @@
 /* crypto/bn/bn_gcd.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -58,7 +58,7 @@
 
 #include <stdio.h>
 #include "cryptlib.h"
-#include "bn.h"
+#include "bn_lcl.h"
 
 #ifndef NOPROTO
 static BIGNUM *euclid(BIGNUM *a, BIGNUM *b);
@@ -100,9 +100,9 @@ BIGNUM *a,*b;
 		if (BN_is_zero(b))
 			break;
 
-		if ((a->d[0]&1))	/* a is odd */
+		if (BN_is_odd(a))
 			{
-			if (b->d[0]&1)  /* b is odd */
+			if (BN_is_odd(b))
 				{
 				if (!BN_sub(a,a,b)) goto err;
 				if (!BN_rshift1(a,a)) goto err;
@@ -118,7 +118,7 @@ BIGNUM *a,*b;
 			}
 		else			/* a is even */
 			{
-			if (b->d[0]&1)	/* b id odd */
+			if (BN_is_odd(b))
 				{
 				if (!BN_rshift1(a,a)) goto err;
 				if (BN_cmp(a,b) < 0)
