@@ -1,5 +1,5 @@
 /* crypto/conf/conf.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
@@ -95,7 +95,7 @@ static CONF_VALUE *get_section();
 
 #define scan_esc(p)	((*(++p) == '\0')?(p):(++p))
 
-char *CONF_version="CONF part of SSLeay 0.8.1b 29-Jun-1998";
+char *CONF_version="CONF part of SSLeay 0.9.0b 29-Jun-1998";
 
 LHASH *CONF_load(h,file,line)
 LHASH *h;
@@ -124,7 +124,9 @@ long *line;
 	in=fopen(file,"rb");
 	if (in == NULL)
 		{
-		SYSerr(SYS_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,get_last_sys_error());
+		ERR_set_error_data(BUF_strdup(file),
+			ERR_TXT_MALLOCED|ERR_TXT_STRING);
 		CONFerr(CONF_F_CONF_LOAD,ERR_R_SYS_LIB);
 		goto err;
 		}
@@ -706,13 +708,13 @@ char *section;
 	v->value=(char *)sk;
 	
 	vv=(CONF_VALUE *)lh_insert(conf,(char *)v);
-#ifndef WIN16
 	if (vv != NULL)
 		{
+#if !defined(NO_STDIO) && !defined(WIN16)
 		fprintf(stderr,"internal fault\n");
+#endif
 		abort();
 		}
-#endif
 	ok=1;
 err:
 	if (!ok)
