@@ -1,5 +1,5 @@
 /* crypto/evp/bio_md.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
@@ -85,7 +85,7 @@ static int md_free();
 
 static BIO_METHOD methods_md=
 	{
-	BIO_TYPE_MEM,"message digest",
+	BIO_TYPE_MD,"message digest",
 	md_write,
 	md_read,
 	NULL, /* md_puts, */
@@ -148,7 +148,7 @@ int outl;
 			}
 		}
 	BIO_clear_retry_flags(b);
-	BIO_set_flags(b,BIO_get_retry_flags(b->next_bio));
+	BIO_copy_next_retry(b);
 	return(ret);
 	}
 
@@ -174,7 +174,7 @@ int inl;
 			}
 		}
 	BIO_clear_retry_flags(b);
-	BIO_set_flags(b,BIO_get_retry_flags(b->next_bio));
+	BIO_copy_next_retry(b);
 	return(ret);
 	}
 
@@ -184,7 +184,7 @@ int cmd;
 long num;
 char *ptr;
 	{
-	EVP_MD_CTX *ctx,*dctx;
+	EVP_MD_CTX *ctx,*dctx,**pctx;
 	EVP_MD **ppmd;
 	EVP_MD *md;
 	long ret=1;
@@ -210,6 +210,21 @@ char *ptr;
 		else
 			ret=0;
 		break;
+	case BIO_C_GET_MD_CTX:
+		if (b->init)
+			{
+			pctx=(EVP_MD_CTX **)ptr;
+			*pctx=ctx;
+			}
+		else
+			ret=0;
+		break;
+	case BIO_C_DO_STATE_MACHINE:
+		BIO_clear_retry_flags(b);
+		ret=BIO_ctrl(b->next_bio,cmd,num,ptr);
+		BIO_copy_next_retry(b);
+		break;
+
 	case BIO_C_SET_MD:
 		md=(EVP_MD *)ptr;
 		EVP_DigestInit(ctx,md);
