@@ -1,17 +1,22 @@
 /* crypto/evp/e_ecb_3d.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -47,22 +58,19 @@
 
 #include <stdio.h>
 #include "cryptlib.h"
-#include "envelope.h"
+#include "evp.h"
 #include "objects.h"
 
 #ifndef NOPROTO
 static void des_ede_init_key(EVP_CIPHER_CTX *ctx, unsigned char *key,
-	unsigned char *iv);
-static void des_ede3_init_enc_key(EVP_CIPHER_CTX *ctx, unsigned char *key,
-	unsigned char *iv);
-static void des_ede3_init_dec_key(EVP_CIPHER_CTX *ctx, unsigned char *key,
-	unsigned char *iv);
+	unsigned char *iv,int enc);
+static void des_ede3_init_key(EVP_CIPHER_CTX *ctx, unsigned char *key,
+	unsigned char *iv,int enc);
 static void des_ede_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
 	unsigned char *in, unsigned int inl);
 #else
 static void des_ede_init_key();
-static void des_ede3_init_enc_key();
-static void des_ede3_init_dec_key();
+static void des_ede3_init_key();
 static void des_ede_cipher();
 #endif
 
@@ -71,17 +79,24 @@ static EVP_CIPHER d_ede_cipher2=
 	NID_des_ede,
 	8,16,0,
 	des_ede_init_key,
-	des_ede_init_key,
 	des_ede_cipher,
+	NULL,
+	sizeof(EVP_CIPHER_CTX)-sizeof((((EVP_CIPHER_CTX *)NULL)->c))+
+		sizeof((((EVP_CIPHER_CTX *)NULL)->c.des_ede)),
+	NULL,
+	NULL,
 	};
 
 static EVP_CIPHER d_ede3_cipher3=
 	{
 	NID_des_ede3,
 	8,24,0,
-	des_ede3_init_enc_key,
-	des_ede3_init_dec_key,
+	des_ede3_init_key,
 	des_ede_cipher,
+	NULL,
+	sizeof(EVP_CIPHER_CTX)-sizeof((((EVP_CIPHER_CTX *)NULL)->c))+
+		sizeof((((EVP_CIPHER_CTX *)NULL)->c.des_ede)),
+	NULL,
 	};
 
 EVP_CIPHER *EVP_des_ede()
@@ -94,10 +109,11 @@ EVP_CIPHER *EVP_des_ede3()
 	return(&d_ede3_cipher3);
 	}
 
-static void des_ede_init_key(ctx,key,iv)
+static void des_ede_init_key(ctx,key,iv,enc)
 EVP_CIPHER_CTX *ctx;
 unsigned char *key;
 unsigned char *iv;
+int enc;
 	{
 	if (key != NULL)
 		{
@@ -109,10 +125,11 @@ unsigned char *iv;
 		}
 	}
 
-static void des_ede3_init_enc_key(ctx,key,iv)
+static void des_ede3_init_key(ctx,key,iv,enc)
 EVP_CIPHER_CTX *ctx;
 unsigned char *key;
 unsigned char *iv;
+int enc;
 	{
 	if (key != NULL)
 		{
@@ -122,19 +139,6 @@ unsigned char *iv;
 		}
 	}
 
-static void des_ede3_init_dec_key(ctx,key,iv)
-EVP_CIPHER_CTX *ctx;
-unsigned char *key;
-unsigned char *iv;
-	{
-	if (key != NULL)
-		{
-		des_set_key((des_cblock *)key,ctx->c.des_ede.ks3);
-		des_set_key((des_cblock *)&(key[8]),ctx->c.des_ede.ks2);
-		des_set_key((des_cblock *)&(key[16]),ctx->c.des_ede.ks1);
-		}
-	}
-
 static void des_ede_cipher(ctx,out,in,inl)
 EVP_CIPHER_CTX *ctx;
 unsigned char *out;
