@@ -1,17 +1,22 @@
 /* crypto/evp/e_ofb_r2.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -49,12 +60,12 @@
 
 #include <stdio.h>
 #include "cryptlib.h"
-#include "envelope.h"
+#include "evp.h"
 #include "objects.h"
 
 #ifndef NOPROTO
 static void rc2_ofb_init_key(EVP_CIPHER_CTX *ctx, unsigned char *key,
-	unsigned char *iv);
+	unsigned char *iv,int enc);
 static void rc2_ofb_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
 	unsigned char *in, unsigned int inl);
 #else
@@ -64,11 +75,15 @@ static void rc2_ofb_cipher();
 
 static EVP_CIPHER r2_ofb_cipher=
 	{
-	NID_rc2_ofb,
+	NID_rc2_ofb64,
 	1,EVP_RC2_KEY_SIZE,8,
 	rc2_ofb_init_key,
-	rc2_ofb_init_key,
 	rc2_ofb_cipher,
+	NULL,
+	sizeof(EVP_CIPHER_CTX)-sizeof((((EVP_CIPHER_CTX *)NULL)->c))+
+		sizeof((((EVP_CIPHER_CTX *)NULL)->c.rc2_ks)),
+	EVP_CIPHER_set_asn1_iv,
+	EVP_CIPHER_get_asn1_iv,
 	};
 
 EVP_CIPHER *EVP_rc2_ofb()
@@ -76,18 +91,20 @@ EVP_CIPHER *EVP_rc2_ofb()
 	return(&r2_ofb_cipher);
 	}
 	
-static void rc2_ofb_init_key(ctx,key,iv)
+static void rc2_ofb_init_key(ctx,key,iv,enc)
 EVP_CIPHER_CTX *ctx;
 unsigned char *key;
 unsigned char *iv;
+int enc;
 	{
-	ctx->c.rc2_cfb.num=0;
+	ctx->num=0;
 
 	if (iv != NULL)
-		memcpy(&(ctx->c.rc2_cfb.oiv[0]),iv,8);
-	memcpy(&(ctx->c.rc2_cfb.iv[0]),&(ctx->c.rc2_cfb.oiv[0]),8);
+		memcpy(&(ctx->oiv[0]),iv,8);
+	memcpy(&(ctx->iv[0]),&(ctx->oiv[0]),8);
 	if (key != NULL)
-		RC2_set_key(&(ctx->c.rc2_cfb.ks),EVP_RC2_KEY_SIZE,key);
+		RC2_set_key(&(ctx->c.rc2_ks),EVP_RC2_KEY_SIZE,key,
+			EVP_RC2_KEY_SIZE*8);
 	}
 
 static void rc2_ofb_cipher(ctx,out,in,inl)
@@ -98,9 +115,9 @@ unsigned int inl;
 	{
 	RC2_ofb64_encrypt(
 		in,out,
-		(long)inl, &(ctx->c.rc2_cfb.ks),
-		&(ctx->c.rc2_cfb.iv[0]),
-		&ctx->c.rc2_cfb.num);
+		(long)inl, &(ctx->c.rc2_ks),
+		&(ctx->iv[0]),
+		&ctx->num);
 	}
 
 #endif
