@@ -1,17 +1,22 @@
 /* rsaref/rsaref.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -53,20 +64,20 @@
 #include "rand.h"
 
 /* 
- * RSAREFerr(RSA_private_decrypt,RE_CONTENT_ENCODING);
- * RSAREFerr(RSA_private_decrypt,RE_DATA);
- * RSAREFerr(RSA_private_decrypt,RE_DIGEST_ALGORITHM);
- * RSAREFerr(RSA_private_decrypt,RE_ENCODING);
- * RSAREFerr(RSA_private_decrypt,RE_KEY);
- * RSAREFerr(RSA_private_decrypt,RE_KEY_ENCODING);
- * RSAREFerr(RSA_private_decrypt,RE_LEN);
- * RSAREFerr(RSA_private_decrypt,RE_MODULUS_LEN);
- * RSAREFerr(RSA_private_decrypt,RE_NEED_RANDOM);
- * RSAREFerr(RSA_private_decrypt,RE_PRIVATE_KEY);
- * RSAREFerr(RSA_private_decrypt,RE_PUBLIC_KEY);
- * RSAREFerr(RSA_private_decrypt,RE_SIGNATURE);
- * RSAREFerr(RSA_private_decrypt,RE_SIGNATURE_ENCODING);
- * RSAREFerr(RSA_private_decrypt,RE_ENCRYPTION_ALGORITHM);
+ * RSAREFerr(RSAREF_F_RSA_REF_BN2BIN,RSAREF_R_CONTENT_ENCODING);
+ * RSAREFerr(RSAREF_F_RSA_REF_PRIVATE_DECRYPT,RSAREF_R_DATA);
+ * RSAREFerr(RSAREF_F_RSA_REF_PRIVATE_ENCRYPT,RSAREF_R_DIGEST_ALGORITHM);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_DECRYPT,RSAREF_R_ENCODING);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_KEY);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_KEY_ENCODING);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_LEN);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_MODULUS_LEN);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_NEED_RANDOM);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_PRIVATE_KEY);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_PUBLIC_KEY);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_SIGNATURE);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_SIGNATURE_ENCODING);
+ * RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,RSAREF_R_ENCRYPTION_ALGORITHM);
  * RSAREFerr(RSAREF_F_RSAREF_BN2BIN,ERR_R_BN_LIB);
  */
 
@@ -77,15 +88,70 @@ static BIGNUM* RSAref_bin2bn(unsigned char* from, BIGNUM * to, int max);
 #endif
 static int RSAref_Public_eay2ref(RSA * from, RSArefPublicKey * to);
 static int RSAref_Private_eay2ref(RSA * from, RSArefPrivateKey * to);
+int RSA_ref_private_decrypt(int len, unsigned char *from,
+	unsigned char *to, RSA *rsa, int padding);
+int RSA_ref_private_encrypt(int len, unsigned char *from,
+	unsigned char *to, RSA *rsa, int padding);
+int RSA_ref_public_encrypt(int len, unsigned char *from,
+	unsigned char *to, RSA *rsa, int padding);
+int RSA_ref_public_decrypt(int len, unsigned char *from,
+	unsigned char *to, RSA *rsa, int padding);
+static int BN_ref_mod_exp(BIGNUM *r,BIGNUM *a,BIGNUM *p,BIGNUM *m, BN_CTX *ctx);
+static int RSA_ref_mod_exp(BIGNUM *r0, BIGNUM *I, RSA *rsa);
 #else
+
 static int RSAref_bn2bin();
 #ifdef undef
 static BIGNUM* RSAref_bin2bn();
 #endif
 static int RSAref_Public_eay2ref();
 static int RSAref_Private_eay2ref();
+static int BN_ref_mod_exp();
+static int RSA_ref_mod_exp();
+int RSA_ref_private_decrypt();
+int RSA_ref_private_encrypt();
+int RSA_ref_public_encrypt();
+int RSA_ref_public_decrypt();
+static int BN_ref_mod_exp();
+static int RSA_ref_mod_exp();
 #endif
 
+static RSA_METHOD rsa_pkcs1_ref_meth={
+	"RSAref PKCS#1 RSA",
+	RSA_ref_public_encrypt,
+	RSA_ref_public_decrypt,
+	RSA_ref_private_encrypt,
+	RSA_ref_private_decrypt,
+	RSA_ref_mod_exp,
+	BN_ref_mod_exp,
+	NULL,
+	NULL,
+	0,
+	NULL,
+	};
+
+RSA_METHOD *RSA_PKCS1_RSAref()
+	{
+	return(&rsa_pkcs1_ref_meth);
+	}
+
+static int RSA_ref_mod_exp(r0, I, rsa)
+BIGNUM *r0;
+BIGNUM *I;
+RSA *rsa;
+	{
+	RSAREFerr(RSAREF_F_RSA_REF_MOD_EXP,ERR_R_SHOULD_NOT_HAVE_BEEN_CALLED);
+	return(0);
+	}
+
+static int BN_ref_mod_exp(r,a,p,m,ctx)
+BIGNUM *r,*a,*p,*m;
+BN_CTX *ctx;
+	{
+	RSAREFerr(RSAREF_F_BN_REF_MOD_EXP,ERR_R_SHOULD_NOT_HAVE_BEEN_CALLED);
+	return(0);
+	}
+
 static int RSAref_bn2bin(from,to,max)
 BIGNUM *from;
 unsigned char *to; /* [max] */
@@ -96,7 +162,7 @@ int max;
 	i=BN_num_bytes(from);
 	if (i > max)
 		{
-		RSAREFerr(RSAREF_F_RSAREF_BN2BIN,RE_LEN);
+		RSAREFerr(RSAREF_F_RSAREF_BN2BIN,RSAREF_R_LEN);
 		return(0);
 		}
 
@@ -186,10 +252,11 @@ RSArefPrivateKey *to;
 	return(1);
 	}
 
-int RSA_private_decrypt(len,from,to,rsa)
+int RSA_ref_private_decrypt(len,from,to,rsa,padding)
 int len;
 unsigned char *from,*to;
 RSA *rsa;
+int padding;
 	{
 	int i,outlen= -1;
 	RSArefPrivateKey RSAkey;
@@ -198,7 +265,7 @@ RSA *rsa;
 		goto err;
 	if ((i=RSAPrivateDecrypt(to,&outlen,from,len,&RSAkey)) != 0)
 		{
-		RSAREFerr(RSAREF_F_RSA_PRIVATE_DECRYPT,i);
+		RSAREFerr(RSAREF_F_RSA_REF_PRIVATE_DECRYPT,i);
 		outlen= -1;
 		}
 err:
@@ -206,10 +273,11 @@ err:
 	return(outlen);
 	}
 
-int RSA_private_encrypt(len,from,to,rsa)
+int RSA_ref_private_encrypt(len,from,to,rsa,padding)
 int len;
 unsigned char *from,*to;
 RSA *rsa;
+int padding;
 	{
 	int i,outlen= -1;
 	RSArefPrivateKey RSAkey;
@@ -218,7 +286,7 @@ RSA *rsa;
 		goto err;
 	if ((i=RSAPrivateEncrypt(to,&outlen,from,len,&RSAkey)) != 0)
 		{
-		RSAREFerr(RSAREF_F_RSA_PRIVATE_ENCRYPT,i);
+		RSAREFerr(RSAREF_F_RSA_REF_PRIVATE_ENCRYPT,i);
 		outlen= -1;
 		}
 err:
@@ -226,10 +294,11 @@ err:
 	return(outlen);
 	}
 
-int RSA_public_decrypt(len,from,to,rsa)
+int RSA_ref_public_decrypt(len,from,to,rsa,padding)
 int len;
 unsigned char *from,*to;
 RSA *rsa;
+int padding;
 	{
 	int i,outlen= -1;
 	RSArefPublicKey RSAkey;
@@ -238,7 +307,7 @@ RSA *rsa;
 		goto err;
 	if ((i=RSAPublicDecrypt(to,&outlen,from,len,&RSAkey)) != 0)
 		{
-		RSAREFerr(RSAREF_F_RSA_PUBLIC_DECRYPT,i);
+		RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_DECRYPT,i);
 		outlen= -1;
 		}
 err:
@@ -246,10 +315,11 @@ err:
 	return(outlen);
 	}
 
-int RSA_public_encrypt(len,from,to,rsa)
+int RSA_ref_public_encrypt(len,from,to,rsa,padding)
 int len;
 unsigned char *from,*to;
 RSA *rsa;
+int padding;
 	{
 	int outlen= -1;
 	int i;
@@ -270,7 +340,7 @@ RSA *rsa;
 		goto err;
 	if ((i=RSAPublicEncrypt(to,&outlen,from,len,&RSAkey,&rnd)) != 0)
 		{
-		RSAREFerr(RSAREF_F_RSA_PUBLIC_DECRYPT,i);
+		RSAREFerr(RSAREF_F_RSA_REF_PUBLIC_ENCRYPT,i);
 		outlen= -1;
 		goto err;
 		}
