@@ -4,13 +4,14 @@ $#ARGV == 0 || die "usage: ssldir.pl /new/path\n";
 @files=('crypto/cryptlib.h',
 	'Makefile.ssl',
 	'tools/c_rehash',
+	'util/mk1mf.pl',
 	);
 
 %cryptlib=(
-	'\sX509_CERT_AREA\s',"#define X509_CERT_AREA\t".'"%s"',
+	'\sX509_CERT_AREA\s',"#define X509_CERT_AREA\t\t".'"%s"',
 	'\sX509_CERT_DIR\s', "#define X509_CERT_DIR\t\t".'"%s/certs"',
-	'\sX509_CERT_FILE\s', "#define X509_CERT_FILE\t".'"%s/cert.pem"',
-	'\sX509_CERT_PRIVATE_DIR\s',"#define X509_PRIVATE_DIR\t".'"%s/private"',
+	'\sX509_CERT_FILE\s', "#define X509_CERT_FILE\t\t".'"%s/cert.pem"',
+	'\sX509_PRIVATE_DIR\s',"#define X509_PRIVATE_DIR\t".'"%s/private"',
 	);
 
 %Makefile_ssl=(
@@ -18,14 +19,17 @@ $#ARGV == 0 || die "usage: ssldir.pl /new/path\n";
 	);
 
 %c_rehash=(
-	'^PATH=',	'PATH=%s/bin:$PATH',
-	'^SSL_DIR=',	'SSL_DIR=%s/certs',
+	'^DIR=',	'DIR=%s',
 	);
 
+%mk1mf=(
+	'^$INSTALLTOP=','$INSTALLTOP="%s";',
+	);
 
 &dofile("crypto/cryptlib.h",$ARGV[0],%cryptlib);
 &dofile("Makefile.ssl",$ARGV[0],%Makefile_ssl);
 &dofile("tools/c_rehash",$ARGV[0],%c_rehash);
+&dofile("util/mk1mf.pl",$ARGV[0],%mk1mf);
 
 sub dofile
 	{
